@@ -30,7 +30,7 @@ OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/lib/Xt/Convert.c,v 3.10 2006/01/09 14:59:20 dawes Exp $ */
+/* $XFree86: xc/lib/Xt/Convert.c,v 3.11 2006/12/10 15:58:24 tsi Exp $ */
 
 /*
 
@@ -501,9 +501,9 @@ void _XtConverterCacheStats(void)
 	    }
 	    (void) fprintf(stdout, "Index: %4d  Entries: %d\n", i, entries);
 	    for (p = cacheHashTable[i]; p; p = p->next) {
-		(void) fprintf(stdout, "    Size: %3d  Refs: %3d  '",
+		(void) fprintf(stdout, "    Size: %3d  Refs: %3ld  '",
 			       p->from.size,
-			       p->has_ext ? CEXT(p)->ref_count : 0);
+			       p->has_ext ? CEXT(p)->ref_count : 0L);
 		(void) fprintf(stdout, "'\n");
 	    }
 	    (void) fprintf(stdout, "\n");
