@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/include/servermd.h,v 3.58 2004/03/21 11:27:06 herrb Exp $ */
+/* $XFree86: xc/programs/Xserver/include/servermd.h,v 3.60 2005/06/29 01:14:12 dawes Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -45,13 +45,13 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $Xorg: servermd.h,v 1.3 2000/08/17 19:53:31 cpqbld Exp $ */
 
 #ifndef SERVERMD_H
 #define SERVERMD_H 1
 
 /*
  * Machine dependent values:
+ *
  * GLYPHPADBYTES should be chosen with consideration for the space-time
  * trade-off.  Padding to 0 bytes means that there is no wasted space
  * in the font bitmaps (both on disk and in memory), but that access of
@@ -111,10 +111,12 @@ SOFTWARE.
  *	define uses unaligned reads for 8-bit BitBLT instead of doing
  *	aligned reads and combining the results with shifts and
  *	logical-ors.  Currently defined for 68020 and vax.
+ *
  *  PLENTIFUL_REGISTERS -
  *	For machines with > 20 registers.  Currently used for
  *	unrolling the text painting code a bit more.  Currently
  *	defined for MIPS.
+ *
  *  SHARED_IDCACHE -
  *	For non-Harvard RISC machines, those which share the same
  *	CPU memory bus for instructions and data.  This unrolls some
@@ -134,26 +136,17 @@ SOFTWARE.
 
 #ifdef __arm32__
 
-#define IMAGE_BYTE_ORDER        LSBFirst
+# define IMAGE_BYTE_ORDER        LSBFirst
+# define BITMAP_BIT_ORDER        LSBFirst
 
-# if defined(XF86MONOVGA) || defined(XF86VGA16) || defined(XF86MONO)
-#  define BITMAP_BIT_ORDER      MSBFirst
-# else
-#  define BITMAP_BIT_ORDER      LSBFirst
-# endif
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16)
-#  define BITMAP_SCANLINE_UNIT  8
-# endif
-
-#define GLYPHPADBYTES           4
-#define GETLEFTBITS_ALIGNMENT   1
-#define LARGE_INSTRUCTION_CACHE
-#define AVOID_MEMORY_READ
+# define GLYPHPADBYTES           4
+# define GETLEFTBITS_ALIGNMENT   1
+# define LARGE_INSTRUCTION_CACHE
+# define AVOID_MEMORY_READ
 
 #endif /* __arm32__ */
 
-#if defined (hpux)
+#if defined(hpux)
 
 #define IMAGE_BYTE_ORDER	MSBFirst
 #define BITMAP_BIT_ORDER	MSBFirst
@@ -317,16 +310,7 @@ SOFTWARE.
 
 #if defined(__alpha) || defined(__alpha__) || defined(__alphaCross)
 # define IMAGE_BYTE_ORDER	LSBFirst	/* Values for the Alpha only */
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16) || defined(XF86MONO)
-#  define BITMAP_BIT_ORDER      MSBFirst
-# else
-#  define BITMAP_BIT_ORDER      LSBFirst
-# endif
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16)
-#  define BITMAP_SCANLINE_UNIT  8
-# endif
+# define BITMAP_BIT_ORDER       LSBFirst
 
 # define GLYPHPADBYTES		4
 # define GETLEFTBITS_ALIGNMENT	1
@@ -370,16 +354,7 @@ SOFTWARE.
 
 #if defined(__ia64__) || defined(ia64)
 # define IMAGE_BYTE_ORDER	LSBFirst
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16) || defined(XF86MONO)
-#  define BITMAP_BIT_ORDER      MSBFirst
-# else
-#  define BITMAP_BIT_ORDER      LSBFirst
-# endif
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16)
-#  define BITMAP_SCANLINE_UNIT  8
-# endif
+# define BITMAP_BIT_ORDER       LSBFirst
 
 # define GLYPHPADBYTES		4
 # define GETLEFTBITS_ALIGNMENT	1
@@ -389,18 +364,9 @@ SOFTWARE.
 
 #endif /* ia64 */
 
-#if defined(__AMD64__) || defined(AMD64) || defined(__amd64__)
+#if defined(__amd64__) || defined(__x86_64__)
 # define IMAGE_BYTE_ORDER	LSBFirst
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16) || defined(XF86MONO)
-#  define BITMAP_BIT_ORDER      MSBFirst
-# else
-#  define BITMAP_BIT_ORDER      LSBFirst
-# endif
-
-# if defined(XF86MONOVGA) || defined(XF86VGA16)
-#  define BITMAP_SCANLINE_UNIT  8
-# endif
+# define BITMAP_BIT_ORDER       LSBFirst
 
 # define GLYPHPADBYTES		4
 # define GETLEFTBITS_ALIGNMENT	1
@@ -408,7 +374,7 @@ SOFTWARE.
 # define FAST_CONSTANT_OFFSET_MODE
 /* ???? */
 # define FAST_UNALIGNED_READS
-#endif /* AMD64 */
+#endif /* __amd64__ || __x86_64__ */
 
 #ifdef stellar
 
@@ -453,17 +419,7 @@ SOFTWARE.
 #endif
 
 #ifndef BITMAP_BIT_ORDER
-# if defined(XF86MONOVGA) || defined(XF86VGA16) || defined(XF86MONO)
-#  define BITMAP_BIT_ORDER      MSBFirst
-# else
-#  define BITMAP_BIT_ORDER      LSBFirst
-# endif
-#endif
-
-#ifndef BITMAP_SCANLINE_UNIT
-# if defined(XF86MONOVGA) || defined(XF86VGA16)
-#  define BITMAP_SCANLINE_UNIT  8
-# endif
+#define BITMAP_BIT_ORDER        LSBFirst
 #endif
 
 #ifndef GLYPHPADBYTES
@@ -520,9 +476,19 @@ SOFTWARE.
 #define IMAGE_BUFSIZE		(64*1024)
 #endif
 
+#if defined(XF86VGA16) || defined(XF86MONO)
+# undef BITMAP_BIT_ORDER	/* Always override this */
+# define BITMAP_BIT_ORDER	MSBFirst
+#endif
+
+#ifdef XF86VGA16
+# undef  BITMAP_SCANLINE_UNIT	/* Always override */
+# define BITMAP_SCANLINE_UNIT	8
+#endif
+
 /* pad scanline to a longword */
 #ifndef BITMAP_SCANLINE_UNIT
-#define BITMAP_SCANLINE_UNIT	32
+# define BITMAP_SCANLINE_UNIT	32
 #endif
 
 #ifndef BITMAP_SCANLINE_PAD
